
/*
	GAudio 2.0.2, (C)2001 by Obraz Studio(R)
	This program is a part of the GAudio 2.0.2 SDK.

	Replay many WAV together with a XM file in background.
*/

	#include <conio.h>
	#include <stdio.h>
	#include "../../include/gaudio.h"

	using namespace gaudio;

#define MAX_NBWAV	16

unsigned	DllVersion;
GAudio		*gAudio;
WAVstream	*WAV[MAX_NBWAV];
TrackerXM	*XM;

void main()
{
	ErrCode err;

	DllVersion = getLibVersion();
	
	char rev = DllVersion & 0xf;
	char sec = (DllVersion >> 4) & 0xf;
	char prim = (DllVersion >> 8) & 0xf;
	printf("GAudio library version %ld.%ld.%ld\n\n", prim, sec, rev);

	printf("This example demonstrates how to load one WAV file and duplicate it\nfor multi replaying which share the same memory ressource.\n");
	printf("Press any key to start ..\n");
	getch();

	gAudio = new GAudio();
	err = gAudio->init(DEVICE_DSOUND, DEVICEFREQ_44K, FALSE);
	if (err != GA_OK) {
		printf(gAudio->getLastErrorString());
		getch();
		delete gAudio;
		return;
	}

	gAudio->installSoftwareMixer();
	gAudio->setAudioStageStatus(AUDIOSTAGE_MSX, TRUE);
	gAudio->setAudioStageStatus(AUDIOSTAGE_TRK, TRUE);

	TrackerParameters XMParams;
	ZeroMemory(&XMParams, sizeof XMParams);
	XMParams.classID = CLSID_TRACKERXM;
	XMParams.fileName = "../../audio bin/tunnel.xm";
	XMParams.flags = FLAG_LOOP;
	XMParams.input = INPUT_DISK;
	XMParams.output = OUTPUT_MIXER;
	XMParams.audioStage = AUDIOSTAGE_TRK;

	err = gAudio->getSoundClass(SOUNDCLASS_XM, &XMParams, (void**)&XM);
	if (err != GA_OK) {
		printf(gAudio->getLastErrorString());
		getch();
		delete gAudio;
		return;
	}

	StreamParameters WAVParams;
	ZeroMemory(&WAVParams, sizeof WAVParams);
	WAVParams.classID = CLSID_WAVSTREAM;
	WAVParams.fileName = "../../audio bin/trek12.wav";
	WAVParams.input = INPUT_DISK;
	WAVParams.audioStage = AUDIOSTAGE_MSX;
	WAVParams.output = OUTPUT_MIXER;

	err = gAudio->getSoundClass(SOUNDCLASS_WAV, &WAVParams, (void**)&WAV[0]);
	if (err != GA_OK) {
		printf(gAudio->getLastErrorString());
		getch();
		delete gAudio;
		return;
	}

	WAV[0]->setVolume(0.3f);
	WAV[0]->setPan(-1.f);
	for (int iWAV = 1; iWAV < MAX_NBWAV; iWAV++) {
		WAV[0]->clone( (Stream**)&WAV[ iWAV ] );
		iWAV&1 ? WAV[ iWAV ]->setPan(1.f) : WAV[ iWAV ]->setPan(-1.f);
	}

	printf("\nPress space bar to start a new sound, other to quit.\n");

	XM->setVolume(0.1f);
	XM->play();

	int actuWAV = 0;
	while(true) {
		char c = getch();
		if (c == ' ') {
			WAV[ actuWAV % MAX_NBWAV ]->stop();
			WAV[ actuWAV % MAX_NBWAV ]->play();
			actuWAV++;
		}
		else break;
	}

	// -----------------------------
	// Note about GAudio destructor:
	// in theory, there is no need to call release() method of each class because GAudio destructor
	// automatically does. However it's quite recommanded to do it for a cleaner and safety program.
	// For this simple example, GAudio destructor will do the job...

	delete gAudio;
}
