
#ifndef GAUDIO2x
#define GAUDIO2x

#ifndef DIRECTSOUND_VERSION
#define DIRECTSOUND_VERSION	0x0800
#endif
#define GAUDIO_VERSION		0x0202

	#include <windows.h>	
	#include <mmsystem.h>
	#include <mmreg.h>
	#include <dsound.h>
	#include <msacm.h>

#define GA_API	_stdcall

#ifndef uchar
typedef unsigned char uchar;
#endif
#ifndef ushort
typedef unsigned short ushort;
#endif

#ifndef TRUE
#define TRUE	-1
#endif
#ifndef FALSE
#define FALSE	0
#endif

#if DIRECTSOUND_VERSION < 0x0800
#define LPDIRECTSOUND8 LPVOID
#define LPDIRECTSOUND3DBUFFER8 LPVOID
#define LPDIRECTSOUND3DLISTENER8 LPVOID
#endif

namespace gaudio
{

const AUDIOSTAGE_ASYNC		= 0x1;
const AUDIOSTAGE_SYNC		= 0x2;

const GETMODE_HEARDTIME		= 0x1;
const GETMODE_WRITETIME		= 0x2;

const CDTRACK_STOP			= 0x1;
const CDTRACK_LOOP			= 0x2;
const CDTRACK_NEXT			= 0x3;

const WRITESAMPLETYPE_ORG	= 0x1;
const WRITESAMPLETYPE_MIXER	= 0x2;

const FLAG_FORWARDLOOP		= 0x1;
const FLAG_PINGPONGLOOP		= 0x2;
const FLAG_LOOP				= 0x1;
const FLAG_MEMPROGRESS		= 0x4;
const FLAG_STANDVOLUME		= 0x8;

const AUDIOSTAGE_SFX		= 0x1;
const AUDIOSTAGE_MSX		= 0x2;
const AUDIOSTAGE_TRK		= 0x3;

const DEVICEFREQ_8K			= 8000;
const DEVICEFREQ_11K		= 11000;
const DEVICEFREQ_16K		= 16000;
const DEVICEFREQ_22K		= 22000;
const DEVICEFREQ_32K		= 32000;
const DEVICEFREQ_44K		= 44100;
const DEVICEFREQ_48K		= 48000;

enum ErrCode
{
	GA_OK = TRUE,
	ERR_FILENOTFOUND = 0x10,
	ERR_BADFORMAT,
	ERR_DSOUNDERROR,
	ERR_SYSERROR,
	ERR_CANTADDCLASS,
	ERR_INVALIDARG,
	ERR_INVALIDPARAM,
	ERR_INVALIDENVIRONMENT,
	ERR_BADCLASSVERSION,
	ERR_UNKOWNERR = 0x10000,
	ERR_FORCEDWORD = 0x7fffffff
};

enum SoundClass
{
	SOUNDCLASS_MOD,
	SOUNDCLASS_S3M,
	SOUNDCLASS_XM,
	SOUNDCLASS_IT,
	SOUNDCLASS_WAV,
	SOUNDCLASS_MPEG,
	SOUNDCLASS_RAW,
	SOUNDCLASS_AUDIOCD,
	SOUNDCLASS_FORCEDWORD = 0x7fffffff
};

enum SampleType
{
	SAMPTYPE_UBYTE,
	SAMPTYPE_WORD,
	SAMPTYPE_FLOAT32,
	SAMPTYPE_FORCEDWORD = 0x7fffffff
};

enum Input
{
	INPUT_DISK,
	INPUT_MEMORY,
	INPUT_FORCEDWORD = 0x7fffffff
};

enum Output
{
	OUTPUT_DSOUND = 1,
	OUTPUT_DSOUNDSOFT,
	OUTPUT_DSOUNDHARD,
	OUTPUT_MIXER,
	OUTPUT_MEMORY,
	OUTPUT_FORCEDWORD = 0x7fffffff
};

enum DeviceType
{
	DEVICE_MUTE,
	DEVICE_SYSTEM,
	DEVICE_DSOUND,
	DEVICE_FORCEDWORD = 0x7fffffff
};

enum NotifyType
{
	NOTIFY_POS,
	NOTIFY_BYTEPOS,
	NOTIFY_TRKNOTE,
	NOTIFY_TRKEFFECT,
	NOTIFY_TRKINSTRU,
	NOTIFY_END,
	NOTIFY_FORCEDWORD = 0x7fffffff
};

enum NotifyTime
{
	NOTIFYTIME_WRITE,
	NOTIFYTIME_HEARD,
	NOTIFYTIME_FORCEDWORD = 0x7fffffff
};

enum SoundStatus
{
	SOUNDSTATUS_STOP,
	SOUNDSTATUS_PAUSE,
	SOUNDSTATUS_PLAY,
	SOUNDSTATUS_FORCEDWORD = 0x7fffffff
};

typedef void NotifyFunction(struct Notify *notify);
typedef void DSPcallback(struct AudioBuffer *audio, unsigned long size);

// ======================
//  ErrorFormat protocol
// ======================
struct ErrorFormat
{
	unsigned		structSize;

	ErrCode			error;
	union		
	{
		unsigned	err;
		HRESULT		hres;
	} extendedError;

	char			*string;
};

// =================================
//  Sound device structure protocol
// =================================
struct DeviceInfo
{
	unsigned			structSize;
	char				*description;
	unsigned			type;
	HWAVEOUT			hwo;
	HWND				hwnd;
	LPDIRECTSOUND		directSound;
	LPDIRECTSOUND8		directSound8;
	GUID				guid;
};

// =================
//	Notify protocol
// =================
struct Notify
{
	unsigned		structSize;

	NotifyType		type;
	NotifyTime		time;
	union
	{
		float		Float;
		int			Int;
	};

	HANDLE				event;
	NotifyFunction		*function;
};

struct AudioBuffer
{
	unsigned		structSize;
	AudioBuffer		*previousBuffer;
	void			*pcm;
	unsigned		nbBlocks;
};

struct TrackerMODinfo
{
	unsigned	structSize;
	char		*name;
	unsigned	length;
	unsigned	nbPatterns;
	unsigned	nbSamples;
	unsigned	nbVoices;
};

struct TrackerS3Minfo
{
	unsigned	structSize;
	char		*name;
	unsigned	length;
	unsigned	nbPatterns;
	unsigned	nbSamples;
	unsigned	nbVoices;
};

struct TrackerXMinfo
{
	unsigned	structSize;
	char		*name;
	unsigned	length;
	unsigned	nbPatterns;
	unsigned	nbInstrus;
	unsigned	nbVoices;
};

struct TrackerITinfo
{
	unsigned	structSize;
	char		*name;
	unsigned	length;
	unsigned	nbPatterns;
	unsigned	nbInstrus;
};

struct WAVinfo
{
	unsigned			structSize;
	unsigned			nbChans;
	unsigned			nbBits;
	unsigned			frequency;
	ushort				formatTag;
	unsigned			dataChunkSize;
	unsigned			avgBytesRate;
	unsigned			unpackedDataSize;
	LPACMDRIVERDETAILS	driverDetails;
};

struct MPEGinfo
{
	unsigned	structSize;
	unsigned	nbChans;
	unsigned	nbBits;
	unsigned	frequency;
	unsigned	dataChunkSize;
	unsigned	unpackedDataSize;
};

struct AudioCDinfo
{
	unsigned	classID;
	unsigned	nbTracks;
	unsigned	totalLength;
	ushort		*tracksLength;
};

struct StreamParameters
{
	unsigned	classID;
	char		*fileName;
	void		*srcMem;
	unsigned	sizeSrcMem;
	unsigned	flags;
	Input		input;
	Output		output;
	unsigned	audioStage;
	unsigned	loopStart, loopEnd;
};

struct RAWParameters
{
	unsigned	classID;
	unsigned	flags;
	Output		output;
	unsigned	audioStage;
	unsigned	nbChannels;
	unsigned	nbBits;
	unsigned	size;
};

struct TrackerParameters
{
	unsigned	classID;
	char		*fileName;
	void		*srcMem;
	unsigned	sizeSrcMem;
	unsigned	flags;
	Input		input;
	Output		output;
	unsigned	audioStage;
	unsigned	nbVirtualChans; // only used for IT (80 is the default value)
};

// ==========================
//	GAudio general interface
// ==========================
const GAUDIO_ID = 0x201;

class GAudio
{
	void		*reserved;
public:
	GAudio();
	~GAudio();
	ErrCode		GA_API init(DeviceType type, unsigned frequency, BOOL bUse3D);
	ErrCode		GA_API init(DeviceInfo *info, unsigned frequency, BOOL bUse3D);
	ErrCode		GA_API getSoundClass(SoundClass sClass, void *params, void **pClass);
	unsigned	GA_API getNbDevice();
	DeviceInfo*	GA_API getDeviceInfo(unsigned numDevice);
	ErrCode		GA_API installSoftwareMixer();
	void		GA_API setGlobalVolume(float vol);
	void		GA_API setGlobalPan(float pan);
	ErrCode		GA_API setAudioStageStatus(unsigned stage, BOOL status);
	unsigned	GA_API createAudioStage(unsigned timeLenght, unsigned type);
	ErrCode		GA_API removeAudioStage(unsigned id);
	ErrCode		GA_API setPreDSPcallback(DSPcallback *function, unsigned long userValue, unsigned long id);
	ErrCode		GA_API setPostDSPcallback(DSPcallback *function, unsigned long userValue, unsigned long id);
	ErrCode		GA_API getLastMixerAudioBlock(AudioBuffer **audioBuffer);
	ErrCode		GA_API retrieveLPDS3DListener(LPDIRECTSOUND3DLISTENER *ds3dListener);
	ErrCode		GA_API retrieveLPDS3DListener8(LPDIRECTSOUND3DLISTENER8 *ds3dListener8);
	ErrCode		GA_API retrieveMixerLPDSBuffer(LPDIRECTSOUNDBUFFER *dsBuffer);
	void		GA_API setDSoundSWBufferFlags(unsigned flags);
	void		GA_API setDSoundHWBufferFlags(unsigned flags);
	ErrCode		GA_API set3DAlgorithm(void *guid);
	ErrCode		GA_API mixerEnterCriticalSection();
	ErrCode		GA_API mixerLeaveCriticalSection();
	ErrorFormat	GA_API getLastError();
	char*		GA_API getLastErrorString();
};

// =========================
//  Common audio interfaces
// =========================
class Audio
{
public:
	ErrCode		virtual GA_API play() = 0;
	ErrCode		virtual GA_API pause() = 0;
	ErrCode		virtual GA_API stop() = 0;
	ErrCode		virtual GA_API getStatus(SoundStatus *status) = 0;
	void		virtual GA_API release() = 0;
};

// ==================
//  Stream interface
// ==================
class Stream : public Audio
{
public:
	void		virtual GA_API setGetMode(unsigned getMode) = 0;
	void		virtual GA_API getStreamPos(unsigned *pos) = 0;
	ErrCode		virtual GA_API seekStreamPos(unsigned pos) = 0;
	void		virtual GA_API setFrequency(unsigned freq) = 0;
	void		virtual GA_API getFrequency(unsigned *freq) = 0;
	void		virtual GA_API setVolume(float vol) = 0;
	void		virtual GA_API getVolume(float *vol) = 0;
	void		virtual GA_API setPan(float pan) = 0;
	void		virtual GA_API getPan(float *pan) = 0;
	ErrCode		virtual GA_API retrievePCMdata(void **data) = 0;
	ErrCode		virtual GA_API retrieveLPDSBuffer(LPDIRECTSOUNDBUFFER *dsBuffer) = 0;
	ErrCode		virtual GA_API retrieveLPDS3DBuffer(LPDIRECTSOUND3DBUFFER *ds3dBuffer) = 0;
	ErrCode		virtual GA_API retrieveLPDS3DBuffer8(LPDIRECTSOUND3DBUFFER8 *ds3dBuffer8) = 0;
	ErrCode		virtual GA_API clone(Stream **stream) = 0;
	ErrCode		virtual GA_API writeSample(unsigned pos, unsigned len, void *data, unsigned type) = 0;
	ErrCode		virtual GA_API setNotifyList(Notify *list, unsigned nb) = 0;
};

const CLSID_WAVSTREAM		= 0x0201;
const CLSID_MPEGSTREAM		= 0x0201;
const CLSID_RAWSTREAM		= 0x0201;

// =================
//  WAVstream class
// =================
class WAVstream : public Stream
{
	void		*reserved;
public:
	ErrCode		GA_API retrieveInfo(WAVinfo *info);
};

// ==================
//  MPEGstream class
// ==================
class MPEGstream : public Stream
{
	void		*reserved;
public:
	ErrCode		GA_API retrieveInfo(MPEGinfo *info);
	ErrCode		GA_API getBitsRate(unsigned *bitsRate);
};

// =================
//  RAWstream class
// =================
class RAWstream : public Stream
{
	void	*reserved;
public:
	void	GA_API clear();
};

// ======================================
//  Tracker interface (MOD, S3M, XM, IT)
// ======================================
class Tracker : public Audio
{
public:
	void		virtual GA_API setGetMode(unsigned getMode) = 0;
	void		virtual GA_API getRow(unsigned *row) = 0;
	void		virtual GA_API getPattern(unsigned *pattern) = 0;
	void		virtual GA_API getPatternLength(unsigned *len) = 0;
	void		virtual GA_API getPos(unsigned *pos) = 0;
	void		virtual GA_API getSpeed(unsigned *speed) = 0;
	void		virtual GA_API getTempo(unsigned tempo) = 0;
	ErrCode		virtual GA_API seekPos(unsigned pos) = 0;
	void		virtual GA_API setVolume(float vol) = 0;
	void		virtual GA_API getVolume(float *vol) = 0;
	ErrCode		virtual GA_API setNotifyList(Notify *list, unsigned nb) = 0;
};

const CLSID_TRACKERMOD	= 0x201;
const CLSID_TRACKERS3M	= 0x201;
const CLSID_TRACKERXM	= 0x201;
const CLSID_TRACKERIT	= 0x201;

// ===========
//  MOD class
// ===========
class TrackerMOD : public Tracker
{
	void		*reserved;
public:
	ErrCode		GA_API retrieveInfo(TrackerMODinfo *info);
	void		GA_API setPanPower(float panPower);
};

// ===========
//  S3M class
// ===========
class TrackerS3M : public Tracker
{
	void		*reserved;
public:
	ErrCode		GA_API retrieveInfo(TrackerS3Minfo *info);
};

// ==========
//  XM class
// ==========
class TrackerXM : public Tracker
{
	void		*reserved;
public:
	ErrCode		GA_API retrieveInfo(TrackerXMinfo *info);
};

// ==========
//  IT class
// ==========
class TrackerIT : public Tracker
{
	void		*reserved;
public:
	ErrCode		GA_API retrieveInfo(TrackerITinfo *info);
};

// ======================
//  AudioCD player class
// ======================
class AudioCD : public Audio
{
	void		*reserved;
public:
	void		GA_API informPos(unsigned track, unsigned min, unsigned sec, unsigned mode);
	ErrCode		GA_API getPos(unsigned *track, unsigned *min, unsigned *sec);
	ErrCode		GA_API open();
	ErrCode		GA_API close();
	BOOL		GA_API isCDpresent();
	void		GA_API retrieveInfo(AudioCDinfo *info);
};

// ====================
//	CONSTANT FUNCTIONS
// ====================
unsigned	GA_API getLibVersion();
unsigned	GA_API getLibID();

} // end of GAudio namespace

#endif
